import sys

from PySide6.QtCore import Slot
from PySide6.QtGui import Qt
from PySide6.QtWidgets import QApplication, QMainWindow, QSplitter, QHBoxLayout, QWidget
from vtkmodules.vtkIOImage import vtkMedicalImageReader2

from gorou.views.vtk_widget import VTKWidget


class GridSplitter(QWidget):
    def __init__(self, reader: vtkMedicalImageReader2):
        super().__init__()
        self.reader = reader
        self.splitter = QSplitter(Qt.Orientation.Vertical)
        self.splitter_top = QSplitter(Qt.Orientation.Horizontal)
        self.splitter_bottom = QSplitter(Qt.Orientation.Horizontal)
        self.splitter.setHandleWidth(0)
        self.splitter_top.setHandleWidth(0)
        self.splitter_bottom.setHandleWidth(0)
        # self.splitter_bottom.setContentsMargins(0, 0, 0, 0)
        self.main_layout = QHBoxLayout()
        self.main_layout.addWidget(self.splitter)
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        self.top_layout = QHBoxLayout()
        self.bottom_layout = QHBoxLayout()
        # reader = vtkMedicalImageReader2()
        # reader.SetFileDimensionality(3)
        # reader.SetDataByteOrderToLittleEndian()
        # reader.SetFileName("D:/Downloads/Z0_Y-0.4_800_1000_0.1_fz1Result0.raw")
        # reader.SetDataExtent(0, 799, 0, 799, 0, 999)
        # reader.Update()
        self.left_top = VTKWidget(self.reader)
        self.left_top.setStyleSheet("background-color: rgb(255, 0, 0);")
        self.right_top = VTKWidget(self.reader)
        self.right_top.image_viewer.SetSliceOrientationToYZ()
        self.right_top.setStyleSheet("background-color: rgb(0, 255, 0);")
        self.left_bottom = VTKWidget(self.reader)
        self.left_bottom.image_viewer.SetSliceOrientationToXZ()
        self.left_bottom.setStyleSheet("background-color: rgb(0, 0, 255)")
        self.right_bottom = VTKWidget(self.reader)
        self.right_bottom.setStyleSheet("background-color: rgb(255, 255, 0)")

        self.splitter_top.addWidget(self.left_top)
        self.splitter_top.addWidget(self.right_top)
        self.splitter_bottom.addWidget(self.left_bottom)
        self.splitter_bottom.addWidget(self.right_bottom)
        self.top_moving = False
        self.bottom_moving = False
        self.splitter_bottom.splitterMoved.connect(self.bottom_splitter_moved)
        self.splitter_top.splitterMoved.connect(self.top_splitter_moved)
        self.splitter.addWidget(self.splitter_top)
        self.splitter.addWidget(self.splitter_bottom)
        self.setLayout(self.main_layout)

    @Slot(int, int)
    def bottom_splitter_moved(self, pos: int, index: int):
        if self.top_moving:
            return
        self.bottom_moving = True
        self.splitter_top.moveSplitter(pos, 1)
        self.bottom_moving = False

    @Slot(int, int)
    def top_splitter_moved(self, pos: int, index: int):
        if self.bottom_moving:
            return
        self.top_moving = True
        self.splitter_bottom.moveSplitter(pos, 1)
        self.top_moving = False

    # def closeEvent(self, event):
    #     super().closeEvent(event)
    #     self.left_top.close()
    #     self.right_top.close()
    #     self.left_bottom.close()
    #     self.right_bottom.close()


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.widget = GridSplitter()
        self.resize(400, 400)

        self.setCentralWidget(self.widget)

    # def closeEvent(self, event):
    #     super().closeEvent(event)
    #     self.widget.close()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    window = MainWindow()
    window.show()
    app.exec()
