from PySide6.QtCore import Slot, QObject, Signal
from PySide6.QtWidgets import QMainWindow, QWidget, QHBoxLayout, QListWidget, QListWidgetItem

from gorou.views.overview import OverviewView
from gorou.views.proxies import Proxies
from gorou.views.settings import SettingsView


class ActiveMenuModel(QObject):
    activeSignal = Signal(str)

    def __init__(self):
        super().__init__()


active_menu = ActiveMenuModel()


class MainContent(QWidget):
    def __init__(self):
        super().__init__()
        self.overview = OverviewView()
        self.settings_view = SettingsView()
        self.proxies_view = Proxies()

        self.main_layout = QHBoxLayout()
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        self.main_layout.addWidget(self.overview)
        self.setLayout(self.main_layout)
        # self.active_model = ActiveMenuModel()

        active_menu.activeSignal.connect(self.set_main_widget)

    @Slot(str)
    def set_main_widget(self, menu_text):
        print(menu_text)
        self.main_layout.itemAt(0).widget().setParent(None)
        if menu_text == "Overview":
            self.main_layout.addWidget(self.overview)
        elif menu_text == "Proxies":
            self.main_layout.addWidget(self.proxies_view)
        elif menu_text == "Settings":
            self.main_layout.addWidget(self.settings_view)


class MainView(QWidget):
    def __init__(self):
        super().__init__()
        self.active_model = ActiveMenuModel()

        main_layout = QHBoxLayout()
        self.menu_list = QListWidget()
        self.main_content = MainContent()
        # self.main_content.setStyleSheet("background-color: rgb(244, 229, 204);")
        main_layout.addWidget(self.menu_list, 1)
        main_layout.addWidget(self.main_content, 4)
        self.setLayout(main_layout)
        self.init_menu()
        self.menu_list.itemClicked.connect(self.menu_clicked)

    def init_menu(self):
        self.menu_list.addItem(QListWidgetItem("Overview"))
        self.menu_list.addItem(QListWidgetItem("Proxies"))
        self.menu_list.addItem(QListWidgetItem("Settings"))
        self.menu_list.setCurrentRow(0)
        # active_menu.activeSignal.emit("Proxies")

    @Slot(QListWidgetItem)
    def menu_clicked(self, item: QListWidgetItem):
        active_menu.activeSignal.emit(item.text())


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("Gorou")
        self.resize(800, 600)
        main_view = MainView()
        self.setCentralWidget(main_view)
