import logging

import serial
from PySide6.QtCore import QThread, Signal, Slot
from PySide6.QtWidgets import QWidget, QListWidget, QVBoxLayout, QListWidgetItem, QHBoxLayout, QPushButton, QDialog, \
    QLabel, QComboBox
from serial import STOPBITS_ONE, EIGHTBITS


class SerialRecvThread(QThread):
    data_received = Signal(str)

    def __init__(self):
        super().__init__()
        self.com = None

    def run(self):
        with serial.Serial(
                'COM3',
                baudrate=9200,
                bytesize=EIGHTBITS,
                stopbits=STOPBITS_ONE,
                timeout=0.1
        ) as ser:
            line = ""
            while True:
                char_b = ser.read()
                if char_b != b'':
                    try:
                        char = char_b.decode('ascii')
                    except UnicodeDecodeError:
                        logging.error("UnicodeDecodeError")
                        line = ""
                        continue
                    line += char
                else:
                    if line != "":
                        self.data_received.emit(line)
                        line = ""


class OverviewView(QWidget):
    def __init__(self):
        super().__init__()
        self.thread = SerialRecvThread()
        main_layout = QVBoxLayout()

        # settings and open
        self.toolbar_layout = QHBoxLayout()
        self.settings_button = QPushButton("Setting")
        self.open_button = QPushButton("Open")
        self.toolbar_layout.addWidget(self.settings_button)
        self.settings_dialog = SettingsDialog()
        self.toolbar_layout.addWidget(self.open_button)
        self.settings_button.clicked.connect(self.open_settings_dialog)

        self.message_list = QListWidget()
        # send_message = QGroup
        main_layout.addLayout(self.toolbar_layout)
        main_layout.addWidget(self.message_list)
        self.setLayout(main_layout)
        self.start_recv()
        self.thread.data_received.connect(self.add_list_item)

    def start_recv(self):
        self.thread.start()

    @Slot(str)
    def add_list_item(self, message: str):
        self.message_list.addItem(QListWidgetItem(message))
        self.message_list.scrollToBottom()

    @Slot()
    def open_settings_dialog(self):
        self.settings_dialog.exec()


class SettingsDialog(QDialog):
    def __init__(self):
        super().__init__()
        self.resize(600, 400)
        self.setWindowTitle("串口设置")
        main_layout = QVBoxLayout()
        port_layout = QHBoxLayout()
        port_label = QLabel("串口号")
        self.port_combo = QComboBox()
        self.port_combo.addItem("COM3")
        self.port_combo.addItem("COM4")
        port_layout.addWidget(port_label)
        port_layout.addWidget(self.port_combo)

        baudrate_layout = QHBoxLayout()
        baudrate_label = QLabel("波特率")
        self.baudrateCombo = QComboBox()
        self.baudrateCombo.addItem("9600")
        self.baudrateCombo.addItem("14400")
        baudrate_layout.addWidget(baudrate_label)
        baudrate_layout.addWidget(self.baudrateCombo)
        main_layout.addLayout(port_layout)
        main_layout.addLayout(baudrate_layout)
        self.setLayout(main_layout)
