from PySide6.QtCore import Slot, QThread, Signal
from PySide6.QtWidgets import QWidget, QVBoxLayout, QPushButton, QFileDialog
from vtkmodules.vtkIOImage import vtkMedicalImageReader2

from gorou.views.grid_splitter import GridSplitter


class RawReader(QThread):
    readSignal = Signal(vtkMedicalImageReader2)

    def __init__(self, filename):
        super().__init__()
        self.filename = filename

    def run(self):
        reader = vtkMedicalImageReader2()
        reader.SetFileDimensionality(3)
        reader.SetDataByteOrderToLittleEndian()
        reader.SetFileName("D:/Downloads/Z0_Y-0.4_800_1000_0.1_fz1Result0.raw")
        reader.SetDataExtent(0, 799, 0, 799, 0, 999)
        reader.Update()
        self.readSignal.emit(reader)


class SettingsView(QWidget):
    def __init__(self):
        super().__init__()
        self.splitter_grid = None
        self.setStyleSheet("background-color: rgb(244, 229, 204);")
        self.main_layout = QVBoxLayout()
        self.readerThread = None
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        open_file = QPushButton("OPEN")
        open_file.clicked.connect(self.pick_file)
        self.main_layout.addWidget(open_file)
        # vtk.vtkMatrix4x4.
        # splitter_grid = GridSplitter()

        # self.vtk_widget.iren.Initialize()
        self.setLayout(self.main_layout)

    @Slot()
    def pick_file(self):
        dialog = QFileDialog()
        filename = dialog.getOpenFileName()
        self.readerThread = RawReader(filename)
        self.readerThread.readSignal.connect(self.add_splitter)
        self.readerThread.start()

    @Slot(vtkMedicalImageReader2)
    def add_splitter(self, reader: vtkMedicalImageReader2):
        self.splitter_grid = GridSplitter(reader)
        self.splitter_grid.setContentsMargins(0, 0, 0, 0)
        self.splitter_grid.setStyleSheet("background-color: rgb(244, 229, 204);")
        self.main_layout.addWidget(self.splitter_grid)
