# noinspection PyUnresolvedReferences
# import vtkmodules.vtkInteractionStyle
# noinspection PyUnresolvedReferences
import vtkmodules.vtkRenderingContextOpenGL2
from PySide6.QtWidgets import QWidget, QVBoxLayout
from vtkmodules.qt.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor
from vtkmodules.vtkIOImage import vtkMedicalImageReader2
from vtkmodules.vtkInteractionImage import vtkImageViewer2
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleImage
from vtkmodules.vtkRenderingCore import vtkRenderer


class VTKWidget(QWidget):
    def __init__(self, reader: vtkMedicalImageReader2, parent=None):
        super().__init__(parent)
        self.content_widget = QWidget()
        self.vtk_widget = QVTKRenderWindowInteractor(self.content_widget)
        self.ren = vtkRenderer()
        self.render_window = self.vtk_widget.GetRenderWindow()
        self.render_window.AddRenderer(self.ren)
        self.iren = self.vtk_widget.GetRenderWindow().GetInteractor()
        self.setContentsMargins(0, 0, 0, 0)

        self.reader = reader

        self.image_viewer = vtkImageViewer2()
        self.image_viewer.SetInputConnection(self.reader.GetOutputPort())
        self.image_viewer.SetRenderWindow(self.render_window)
        self.image_viewer.SetSliceOrientationToXY()
        self.render_window.SetSize(800, 800)
        self.interactor_style = vtkInteractorStyleImage()
        self.iren.SetInteractorStyle(self.interactor_style)

        self.interactor_style.AddObserver("MouseWheelForwardEvent", self.mouse_wheel_forward_event)
        self.interactor_style.AddObserver("MouseWheelBackwardEvent", self.mouse_wheel_backward_event)

        self.main_layout = QVBoxLayout()
        self.main_layout.addWidget(self.content_widget)
        self.setLayout(self.main_layout)
        self.image_viewer.Render()
        self.iren.Initialize()

    def set_reader(self, reader: vtkMedicalImageReader2):
        self.reader = reader

    def mouse_wheel_forward_event(self, a, b):
        if self.image_viewer.GetSlice() < self.image_viewer.GetSliceMax():
            print(self.image_viewer.GetSlice())
            self.image_viewer.SetSlice(self.image_viewer.GetSlice() + 1)
            self.image_viewer.Render()

    def mouse_wheel_backward_event(self, a, b):
        if self.image_viewer.GetSlice() > self.image_viewer.GetSliceMin():
            print(self.image_viewer.GetSlice())
            self.image_viewer.SetSlice(self.image_viewer.GetSlice() - 1)
            self.image_viewer.Render()

    def closeEvent(self, event):
        super().closeEvent(event)
        self.vtk_widget.Finalize()
